@@ -1,3 +1,9 @@
+/* config.c */
+extern void applyconfig(void);
+extern void saveconfig(void);
+extern void configdialog(GtkWidget *widget);
+extern void load_defaults(void);
+extern void save_defaults(void);
 /* gx11.c */
 extern void do_network(void);
 extern void event_loop(void);
@@ -12,6 +18,7 @@ extern void draw_color_info(int colr, const char *buf);
 extern void draw_stats(int redraw);
 extern void create_stat_bar(GtkWidget *mtable, gint row, gchar *label, gint bar, GtkWidget **plabel);
 extern void draw_message_window(int redraw);
+extern void draw_all_list(itemlist *l);
 extern void open_container(item *op);
 extern void close_container(item *op);
 extern void count_callback(GtkWidget *widget, GtkWidget *entry);
@@ -20,11 +27,8 @@ extern void draw_lists(void);
 extern void set_show_icon(char *s);
 extern void set_show_weight(char *s);
 extern void aboutdialog(GtkWidget *widget);
-extern void applyconfig(void);
-extern void saveconfig(void);
 extern void cclist_button_event(GtkWidget *gtklist, gint row, gint column, GdkEventButton *event);
 extern void disconnect(GtkWidget *widget);
-extern void configdialog(GtkWidget *widget);
 extern void chelpdialog(GtkWidget *widget);
 extern void shelpdialog(GtkWidget *widget);
 extern void new_menu_pickup(GtkWidget *button, int val);
@@ -57,30 +61,23 @@ extern void x_set_echo(void);
 extern int do_timeout(void);
 extern void draw_magic_map(void);
 extern void magic_map_flash_pos(void);
+extern void get_window_coord(GtkWidget *win, int *x, int *y, int *wx, int *wy, int *w, int *h);
 extern void save_winpos(void);
 extern void command_show(char *params);
 extern void set_window_pos(void);
 extern int init_windows(int argc, char **argv);
 extern void display_map_doneupdate(int redraw);
 extern void display_map_newmap(void);
-extern int display_usebitmaps(void);
-extern int display_usexpm(void);
-extern int display_usepng(void);
-extern int display_willcache(void);
 extern void resize_map_window(int x, int y);
-extern void display_newpng(long face, uint8 *buf, long buflen);
 extern void display_map_startupdate(void);
 extern char *get_metaserver(void);
-extern void load_defaults(void);
-extern void save_defaults(void);
 extern int main(int argc, char *argv[]);
 /* image.c */
-extern void requestface(int pnum, char *facename, char *facepath);
-extern int create_and_rescale_image_from_data(int pixmap_num, uint8 *rgba_data, int width, int height);
-extern void finish_face_cmd(int pnum, uint32 checksum, int has_sum, char *face);
-extern int ReadImages(void);
-extern int find_face_in_private_cache(char *face, int checksum);
+extern int create_and_rescale_image_from_data(Cache_Entry *ce, int pixmap_num, uint8 *rgba_data, int width, int height);
+extern int associate_cache_entry(Cache_Entry *ce, int pixnum);
 extern void reset_image_data(void);
+extern void image_update_download_status(int start, int end, int total);
+extern void get_map_image_size(int face, uint8 *w, uint8 *h);
 /* keys.c */
 extern void init_keys(void);
 extern void bind_key(char *params);
@@ -95,21 +92,22 @@ extern void ckeyclear(void);
 /* map.c */
 extern void allocate_map(struct Map *new_map, int ax, int ay);
 extern void reset_map(void);
-extern void display_map_clearcell(long x, long y);
 extern void print_darkness(void);
 extern void print_map(void);
 extern void set_map_darkness(int x, int y, uint8 darkness);
-extern void set_map_face(int x, int y, int layer, int face);
-extern void display_map_addbelow(long x, long y, long face);
 extern void display_mapscroll(int dx, int dy);
 extern void reset_map_data(void);
-extern void gtk_draw_map(void);
+extern void gtk_draw_map(int redraw);
 /* png.c */
 extern uint8 *png_to_data(uint8 *data, int len, uint32 *width, uint32 *height);
 extern uint8 *rescale_rgba_data(uint8 *data, int *width, int *height, int scale);
 extern int rgba_to_gdkpixmap(GdkWindow *window, uint8 *data, int width, int height, GdkPixmap **pix, GdkBitmap **mask, GdkColormap *colormap);
 extern int png_to_gdkpixmap(GdkWindow *window, uint8 *data, int len, GdkPixmap **pix, GdkBitmap **mask, GdkColormap *colormap);
 /* sdl.c */
+extern void init_SDL(GtkWidget *sdl_window, int just_lightmap);
+extern void do_sdl_per_pixel_lighting(int x, int y, int mx, int my);
+extern void sdl_gen_map(int redraw);
+extern void sdl_mapscroll(int dx, int dy);
 /* sound.c */
 extern void signal_pipe(int i);
 extern int init_sounds(void);
